@@ -65,7 +65,7 @@ make_boost_tree_lightgbm <- function() {
       predictor_indicators = "none",
       compute_intercept = FALSE,
       remove_intercept = FALSE,
-      allow_sparse_x = FALSE
+      allow_sparse_x = TRUE
     )
   )
 
@@ -111,7 +111,7 @@ make_boost_tree_lightgbm <- function() {
       predictor_indicators = "none",
       compute_intercept = FALSE,
       remove_intercept = FALSE,
-      allow_sparse_x = FALSE
+      allow_sparse_x = TRUE
     )
   )
 
